@@ -722,26 +722,31 @@ plugin_command_list(GList *tmp_list)
 	return tmp_list;
 }
 
+void
+plugin_command_foreach (session *sess, void *userdata,
+			void (*cb) (session *sess, void *userdata, char *name, char *help))
+{
+	GSList *list;
+	xchat_hook *hook;
+
+	list = hook_list;
+	while (list)
+	{
+		hook = list->data;
+		if (hook->type == HOOK_COMMAND && hook->name[0])
+		{
+			cb (sess, userdata, hook->name, hook->help_text);
+		}
+		list = list->next;
+	}
+}
+
 int
 plugin_show_help (session *sess, char *cmd)
 {
 	GSList *list;
 	xchat_hook *hook;
 
-	/* show all help commands */
-	if (cmd == NULL)
-	{
-		list = hook_list;
-		while (list)
-		{
-			hook = list->data;
-			if (hook->type == HOOK_COMMAND)
-				PrintText (sess, hook->name);
-			list = list->next;
-		}
-		return 1;
-	}
-
 	list = plugin_hook_find (hook_list, HOOK_COMMAND, cmd);
 	if (list)
 	{
@@ -1009,6 +1014,9 @@ xchat_get_info (xchat_plugin *ph, const char *id)
 	case 0x14f51cd8: /* version */
 		return VERSION;
 
+	case 0x506d600b: /* native win_ptr */
+		return fe_gui_info_ptr (sess, 0);
+
 	case 0x6d3431b5: /* win_status */
 		switch (fe_gui_info (sess, 0))	/* check window status */
 		{
@@ -1348,7 +1356,9 @@ xchat_list_int (xchat_plugin *ph, xchat_list *xlist, const char *name)
 		case 0xd1b:	/* id */
 			return ((struct session *)data)->server->id;
 		case 0x5cfee87:	/* flags */
-			tmp = ((struct session *)data)->server->have_whox;   /* bit 4 */
+			tmp = ((struct session *)data)->server->have_idmsg;   /* bit 5 */
+			tmp <<= 1;
+			tmp |= ((struct session *)data)->server->have_whox;  /* 4 */
 			tmp <<= 1;
 			tmp |= ((struct session *)data)->server->end_of_motd;/* 3 */
 			tmp <<= 1;
