@@ -89,7 +89,7 @@ log_create_filename (char *buf, char *servname, char *channame, char *netname)
 #else
 				/* win32 can't handle filenames with \|/>< characters */
 				if (*tmp == '\\' || *tmp == '|' || *tmp == '/' ||
-					 *tmp == '>' || *tmp == '<')
+					 *tmp == '>' || *tmp == '<'  || *tmp == ':' || *tmp == '\"' || *tmp == '*' || *tmp == '?')
 #endif
 					*tmp = '_';
 			}
@@ -197,7 +197,7 @@ log_open (session *sess)
 		snprintf (message, sizeof (message),
 					_("* Can't open log file(s) for writing. Check the\n" \
 					  "  permissions on %s/xchatlogs"), get_xdir_utf8 ());
-		fe_message (message, TRUE);
+		fe_message (message, FE_MSG_WAIT | FE_MSG_ERROR);
 
 		log_error = TRUE;
 	}
@@ -427,6 +427,7 @@ static char * const pevt_join_help[] = {
 static char * const pevt_chanaction_help[] = {
 	N_("Nickname"),
 	N_("The action"),
+	N_("Mode char"),
 };
 
 static char * const pevt_chanmsg_help[] = {
@@ -726,6 +727,7 @@ static char * const pevt_sconnect_help[] = {
 
 static char * const pevt_generic_nick_help[] = {
 	N_("Nickname"),
+	N_("Server Name")
 };
 
 static char * const pevt_chanmodes_help[] = {
@@ -932,7 +934,7 @@ pevent_make_pntevts ()
 		{
 			snprintf (out, sizeof (out),
 						 _("Error parsing event %s.\nLoading default."), te[i].name);
-			fe_message (out, FALSE);
+			fe_message (out, FE_MSG_WARN);
 			free (pntevts_text[i]);
 			len = strlen (te[i].def) + 1;
 			pntevts_text[i] = malloc (len);
@@ -1274,7 +1276,7 @@ pevt_build_string (const char *input, char **output, int *max_arg)
 		}
 		if (ii == len)
 		{
-			fe_message ("String ends with a $", FALSE);
+			fe_message ("String ends with a $", FE_MSG_WARN);
 			return 1;
 		}
 		d = i[ii++];
@@ -1302,10 +1304,10 @@ pevt_build_string (const char *input, char **output, int *max_arg)
 			continue;
 
 		 a_len_error:
-			fe_message ("String ends in $a", FALSE);
+			fe_message ("String ends in $a", FE_MSG_WARN);
 			return 1;
 		 a_range_error:
-			fe_message ("$a value is greater than 255", FALSE);
+			fe_message ("$a value is greater than 255", FE_MSG_WARN);
 			return 1;
 		}
 		if (d == 't')
@@ -1328,7 +1330,7 @@ pevt_build_string (const char *input, char **output, int *max_arg)
 		if (d < '1' || d > '9')
 		{
 			snprintf (o, sizeof (o), "Error, invalid argument $%c\n", d);
-			fe_message (o, FALSE);
+			fe_message (o, FE_MSG_WARN);
 			return 1;
 		}
 		d -= '0';
@@ -1585,7 +1587,7 @@ sound_play (const char *file)
 	} else
 	{
 		snprintf (buf, sizeof (buf), _("Cannot read sound file:\n%s"), wavfile);
-		fe_message (buf, FALSE);
+		fe_message (buf, FE_MSG_ERROR);
 	}
 
 	g_free (file_fs);
