@@ -164,7 +164,7 @@ editlist_gui_save (GtkWidget * igad)
 	char buf[512];
 	char *a, *b;
 
-	snprintf (buf, sizeof buf, "%s/%s", get_xdir (), editlist_file);
+	snprintf (buf, sizeof buf, "%s/%s", get_xdir_fs (), editlist_file);
 	fh = open (buf, O_TRUNC | O_WRONLY | O_CREAT | OFLAGS, 0600);
 	if (fh != -1)
 	{
@@ -236,7 +236,7 @@ static void
 editlist_gui_help (GtkWidget * igad)
 {
 /*	if (editlist_help)*/
-		gtkutil_simpledialog (editlist_help);
+		fe_message (editlist_help, FE_MSG_INFO);
 }
 
 static void
@@ -296,14 +296,21 @@ editlist_gui_close (void)
 }
 
 void
-editlist_gui_open (GSList * list, char *title, char *wmclass, char *file,
-						 char *help)
+editlist_gui_open (char *title1, char *title2, GSList * list, char *title, char *wmclass,
+						 char *file, char *help)
 {
 	gchar *titles[2];
 	GtkWidget *vbox, *hbox, *button;
 
-	titles[0] = _("Name");
-	titles[1] = _("Command");
+	if (title1)
+	{
+		titles[0] = title1;
+		titles[1] = title2;
+	} else
+	{
+		titles[0] = _("Name");
+		titles[1] = _("Command");
+	}
 
 	if (editlist_gui_window)
 	{
@@ -353,11 +360,11 @@ editlist_gui_open (GSList * list, char *title, char *wmclass, char *file,
 	gtk_box_pack_end (GTK_BOX (vbox), hbox, 0, 0, 0);
 	gtk_widget_show (hbox);
 
-	button = gtkutil_button (hbox, GTK_STOCK_NEW, 0, editlist_gui_addnew,
+	button = gtkutil_button (hbox, GTK_STOCK_ADD, 0, editlist_gui_addnew,
 									 0, _("Add New"));
 	gtk_widget_set_usize (button, 100, 0);
 
-	button = gtkutil_button (hbox, GTK_STOCK_DELETE, 0, editlist_gui_delete,
+	button = gtkutil_button (hbox, GTK_STOCK_REMOVE, 0, editlist_gui_delete,
 									 0, _("Delete"));
 	gtk_widget_set_usize (button, 100, 0);
 
@@ -380,7 +387,7 @@ editlist_gui_open (GSList * list, char *title, char *wmclass, char *file,
 	gtk_box_pack_end (GTK_BOX (vbox), hbox, 0, 0, 0);
 	gtk_widget_show (hbox);
 
-	editlist_gui_entry_name = gtk_entry_new_with_max_length (40);
+	editlist_gui_entry_name = gtk_entry_new_with_max_length (82);
 	gtk_widget_set_usize (editlist_gui_entry_name, 96, 0);
 	gtk_signal_connect (GTK_OBJECT (editlist_gui_entry_name), "changed",
 							  GTK_SIGNAL_FUNC (editlist_gui_handle_name), 0);
