@@ -51,6 +51,7 @@
 #endif
 
 #include "../common/xchat.h"
+#include "../common/fe.h"
 #include "gtkutil.h"
 #include "pixmaps.h"
 
@@ -70,22 +71,6 @@ struct file_req
 static char last_dir[256] = "";
 
 
-GtkWidget *
-gtkutil_simpledialog (char *msg)
-{
-	GtkWidget *dialog;
-
-	dialog = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_INFO, GTK_BUTTONS_OK,
-						 "%s", msg);
-	g_signal_connect (G_OBJECT (dialog), "response",
-							G_CALLBACK (gtk_widget_destroy), 0);
-	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
-	gtk_widget_show (dialog);
-
-	return dialog;
-}
-
 static void
 gtkutil_file_req_destroy (GtkWidget * wid, struct file_req *freq)
 {
@@ -140,14 +125,20 @@ gtkutil_check_file (char *file, struct file_req *freq)
 		/* convert to UTF8. It might be converted back to locale by
 			server.c's g_convert */
 		utf8_file = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
-		freq->callback (freq->userdata, utf8_file);
-		g_free (utf8_file);
+		if (utf8_file)
+		{
+			freq->callback (freq->userdata, utf8_file);
+			g_free (utf8_file);
+		} else
+		{
+			fe_message ("Filename encoding is corrupt.", FE_MSG_ERROR);
+		}
 	} else
 	{
 		if (freq->write)
-			gtkutil_simpledialog (_("Cannot write to that file."));
+			fe_message (_("Cannot write to that file."), FE_MSG_ERROR);
 		else
-			gtkutil_simpledialog (_("Cannot read that file."));
+			fe_message (_("Cannot read that file."), FE_MSG_ERROR);
 	}
 
 	return 1;
@@ -236,6 +227,7 @@ gtkutil_file_req (char *title, void *callback, void *userdata, char *filter,
 {
 	struct file_req *freq;
 	GtkWidget *dialog;
+	extern char *get_xdir_fs (void);
 
 #if GTK_CHECK_VERSION(2,4,0)
 	/* == GTK 2.4 uses the new GtkFileChooser, which is much nicer! */
@@ -257,7 +249,7 @@ gtkutil_file_req (char *title, void *callback, void *userdata, char *filter,
 		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), last_dir);
 	if (flags & FRF_ADDFOLDER)
 		gtk_file_chooser_add_shortcut_folder (GTK_FILE_CHOOSER (dialog),
-														  (char *)get_xdir_fs (), NULL);
+														  get_xdir_fs (), NULL);
 
 	freq = malloc (sizeof (struct file_req));
 	freq->dialog = dialog;
@@ -538,10 +530,36 @@ gtkutil_clist_selection (GtkWidget * clist)
 	return -1;
 }
 
+static int
+int_compare (const int * elem1, const int * elem2)
+{
+	return (*elem1) - (*elem2);
+}
+
+int
+gtkutil_clist_multiple_selection (GtkWidget * clist, int ** rows, const int max_rows)
+{
+	int i = 0;
+	GList *tmp_clist;
+	*rows = malloc (sizeof (int) * max_rows );
+	memset( *rows, -1, max_rows * sizeof(int) );
+
+	for( tmp_clist = GTK_CLIST(clist)->selection;
+			tmp_clist && i < max_rows; tmp_clist = tmp_clist->next, i++)
+	{
+		(*rows)[i] = GPOINTER_TO_INT( tmp_clist->data );
+	}
+	qsort(*rows, i, sizeof(int), (void *)int_compare);
+	return i;
+
+}
+
 void
 add_tip (GtkWidget * wid, char *text)
 {
-	GtkTooltips *tip = gtk_tooltips_new ();
+	static GtkTooltips *tip = NULL;
+	if (!tip)
+		tip = gtk_tooltips_new ();
 	gtk_tooltips_set_tip (tip, wid, text, 0);
 }
 
@@ -567,6 +585,9 @@ gtkutil_window_new (char *title, char *role, int width, int height, int flags)
 
 	win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	gtkutil_set_icon (win);
+#ifdef WIN32
+	gtk_window_set_wmclass (GTK_WINDOW (win), "XChat", "xchat");
+#endif
 	gtk_window_set_title (GTK_WINDOW (win), title);
 	gtk_window_set_default_size (GTK_WINDOW (win), width, height);
 	gtk_window_set_role (GTK_WINDOW (win), role);
